@@ -18,6 +18,8 @@ from typing import (
     NamedTuple,
     Literal,
     cast,
+    Sequence,
+    FrozenSet,
 )
 
 from debputy import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR
@@ -30,7 +32,11 @@ from debputy.commands.debputy_cmd.context import (
     ROOT_COMMAND,
     CommandArg,
 )
-from debputy.commands.debputy_cmd.output import _stream_to_pager, _output_styling
+from debputy.commands.debputy_cmd.output import (
+    _stream_to_pager,
+    _output_styling,
+    MarkdownOutputStyle,
+)
 from debputy.commands.debputy_cmd.plugin_cmds import lookup_pmr_rule
 from debputy.dh_migration.migrators import MIGRATORS
 from debputy.exceptions import (
@@ -43,10 +49,11 @@ from debputy.exceptions import (
     DebputyRuntimeErrorWithPreamble,
 )
 from debputy.highlevel_manifest import HighLevelManifest
-from debputy.manifest_parser.parser_doc import render_rule
+from debputy.manifest_parser.parser_doc import render_rule, render_type_mapping
 from debputy.package_build.assemble_deb import (
     assemble_debs,
 )
+from debputy.packages import BinaryPackage
 from debputy.plugin.api.spec import (
     INTEGRATION_MODE_DH_DEBPUTY_RRR,
     DebputyIntegrationMode,
@@ -102,7 +109,6 @@ from debputy.util import (
     integrated_with_debhelper,
     PRINT_BUILD_SYSTEM_COMMAND,
     PRINT_COMMAND,
-    manifest_format_doc,
 )
 
 
@@ -693,6 +699,11 @@ def _process_template(context: CommandContext) -> None:
         _error("This sub-command requires a compatible version of python3-jinja2")
 
     feature_set = context.load_plugins()
+    markdown_color_base = MarkdownOutputStyle()
+    manifest_parser = context.manifest_parser()
+
+    type_mappings = context.load_plugins().mapped_types
+    type_mappings_by_name = {t.__name__: type_mappings[t] for t in type_mappings}
 
     with open(template_file) as fd:
         jinja_template = Template(
@@ -706,21 +717,48 @@ def _process_template(context: CommandContext) -> None:
             comment_end_string="#>",
         )
 
-    def _render_pmr(rule_name: str, heading: str) -> str:
+    def _render_pmr(rule_name: str, heading_level: int) -> str:
         pmr_rule = lookup_pmr_rule(feature_set, rule_name)
         return render_rule(
             pmr_rule.rule_name,
             pmr_rule.parser,
             pmr_rule.plugin_metadata,
+            markdown_color_base,
             is_root_rule=pmr_rule.is_root_rule,
-            heading=heading,
+            base_heading_level=heading_level,
             include_ref_doc_link=False,
             include_alt_format=False,
             manifest_format_url="",
         )
 
+    def _render_tm(type_mapping_name: str, heading_level: int) -> str:
+        type_mapping = type_mappings_by_name.get(type_mapping_name)
+        if type_mapping is None:
+            _error(f"No type mapping named: {type_mapping_name}")
+
+        return render_type_mapping(
+            type_mapping,
+            markdown_color_base,
+            manifest_parser,
+            base_heading_level=heading_level,
+            recover_from_broken_examples=False,
+        )
+
+    def _render_all_tms(heading_level: int) -> str:
+        r = []
+        for tm_name in sorted(type_mappings_by_name):
+            r.append(_render_tm(tm_name, heading_level))
+        return "\n\n".join(r)
+
     with open(output_file, "w") as fd:
-        fd.write(jinja_template.render(render_pmr=_render_pmr))
+        r = jinja_template.render(
+            render_pmr=_render_pmr,
+            render_tm=_render_tm,
+            render_all_tms=_render_all_tms,
+        )
+        fd.write(r)
+        if not r.endswith("\n"):
+            fd.write("\n")
 
 
 @internal_commands.register_subcommand(
@@ -773,11 +811,13 @@ def _dpkg_build_driver_integration(context: CommandContext) -> None:
     elif task_name in ("build", "build-indep", "build-arch"):
         perform_builds(context, manifest)
     elif task_name in ("binary", "binary-indep", "binary-arch"):
-        perform_builds(context, manifest)
+        build_system_install_dirs: List[Tuple[str, FrozenSet[BinaryPackage]]] = []
+        perform_builds(context, manifest, build_system_install_dirs)
         assemble(
             context,
             manifest,
             INTEGRATION_MODE_FULL,
+            build_system_install_dirs,
             debug_materialization=log_level is not None,
         )
     else:
@@ -830,6 +870,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
         context,
         manifest,
         integration_mode,
+        (),
         debug_materialization=log_level is not None,
     )
 
@@ -838,6 +879,7 @@ def assemble(
     context: CommandContext,
     manifest: HighLevelManifest,
     integration_mode: DebputyIntegrationMode,
+    build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
     *,
     debug_materialization: bool = False,
 ) -> None:
@@ -845,7 +887,10 @@ def assemble(
     source_version = manifest.source_version()
     is_native = "-" not in source_version
     is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
-    package_data_table = manifest.perform_installations(integration_mode)
+    package_data_table = manifest.perform_installations(
+        integration_mode,
+        build_system_install_dirs,
+    )
     if not is_dh_rrr_only_mode:
         for dctrl_bin in manifest.active_packages:
             package = dctrl_bin.name
@@ -990,6 +1035,7 @@ def _annotate_debian_directory(context: CommandContext) -> None:
 
     result = scan_debian_dir(
         feature_set,
+        context.source_package(),
         binary_packages,
         context.debian_dir,
     )
