@@ -1,11 +1,17 @@
 import asyncio
+import os.path
 import random
 import textwrap
 from argparse import BooleanOptionalAction
+from typing import TYPE_CHECKING
 
 from debputy.commands.debputy_cmd.context import ROOT_COMMAND, CommandContext, add_arg
 from debputy.lsp.lsp_reference_keyword import ALL_PUBLIC_NAMED_STYLES
-from debputy.util import _error
+from debputy.util import _error, _info
+
+if TYPE_CHECKING:
+    from debputy.lsp.debputy_ls import DebputyLanguageServer
+
 
 _EDITOR_SNIPPETS = {
     "emacs": "emacs+eglot",
@@ -69,7 +75,7 @@ _EDITOR_SNIPPETS = {
         #   Feel free to provide one :)
         let g:ycm_language_server = [
           \\   { 'name': 'debputy',
-          \\     'filetypes': [ 'debcontrol', 'debcopyright', 'debchangelog', 'make', 'yaml'],
+          \\     'filetypes': [ 'debcontrol', 'debcopyright', 'debchangelog', 'autopkgtest', 'make', 'yaml'],
           \\     'cmdline': [ 'debputy', 'lsp', 'server', '--ignore-language-ids' ]
           \\   },
           \\ ]
@@ -96,7 +102,7 @@ _EDITOR_SNIPPETS = {
 
         if executable('debputy')
             lspServers->add({
-                filetype: ['debcontrol', 'debcopyright', 'debchangelog', 'make', 'yaml'],
+                filetype: ['debcontrol', 'debcopyright', 'debchangelog', 'autopkgtest', 'make', 'yaml'],
                 path: 'debputy',
                 args: ['lsp', 'server', '--ignore-language-ids']
             })
@@ -128,6 +134,13 @@ lsp_command = ROOT_COMMAND.add_dispatching_subcommand(
     help_description="Language server related subcommands",
 )
 
+lsp_internal_commands = lsp_command.add_dispatching_subcommand(
+    "internal-command",
+    dest="internal_command",
+    metavar="command",
+    help_description="Commands used for internal purposes. These are implementation details and subject to change",
+)
+
 
 @lsp_command.register_subcommand(
     "server",
@@ -385,6 +398,116 @@ def reformat_cmd(context: CommandContext) -> None:
     perform_reformat(context, named_style=context.parsed_args.named_style)
 
 
+@lsp_internal_commands.register_subcommand(
+    "semantic-tokens",
+    log_only_to_stderr=True,
+    help_description="Test semantic tokens",
+    argparser=[
+        add_arg(
+            "filename",
+            help="Filename to annotate",
+        ),
+        add_arg(
+            "--language-id",
+            action="store",
+            type=str,
+            default="",
+            help="Choose the language id",
+        ),
+    ],
+)
+def lsp_test_semantic_tokens_cmd(context: CommandContext) -> None:
+    parsed_args = context.parsed_args
+    filename = parsed_args.filename
+
+    debputy_ls = _load_test_lsp(context)
+
+    with open(filename) as fd:
+        data = fd.read()
+
+    doc_uri = f"file://{os.path.abspath(filename)}"
+
+    _fake_initialize(
+        debputy_ls,
+        doc_uri,
+        data,
+        language_id=parsed_args.language_id,
+    )
+
+    from debputy.lsp.lsp_dispatch import semantic_tokens_full
+    from debputy.lsp.lsp_test_support import resolve_semantic_tokens
+    import lsprotocol.types as types
+    import asyncio
+
+    result = asyncio.get_event_loop().run_until_complete(
+        semantic_tokens_full(
+            debputy_ls,
+            types.SemanticTokensParams(types.TextDocumentIdentifier(doc_uri)),
+        )
+    )
+
+    if result is None:
+        _info("No semantic were tokens provided")
+        return
+
+    lines = data.splitlines(keepends=True)
+    tokens = resolve_semantic_tokens(
+        lines,
+        result,
+    )
+    _info(f"Identified {len(tokens)}")
+    for token in tokens:
+        print(f"{filename}:{token.range}: {token.token_name} {token.value!r}")
+
+
+def _load_test_lsp(
+    context: CommandContext,
+) -> "DebputyLanguageServer":
+    from debputy.lsp.lsp_self_check import assert_can_start_lsp
+
+    assert_can_start_lsp()
+
+    from debputy.lsp.lsp_features import (
+        ensure_lsp_features_are_loaded,
+    )
+    from debputy.lsp.lsp_dispatch import DEBPUTY_LANGUAGE_SERVER
+
+    ensure_lsp_features_are_loaded()
+    debputy_language_server = DEBPUTY_LANGUAGE_SERVER
+    debputy_language_server.plugin_feature_set = context.load_plugins()
+    debputy_language_server.dctrl_parser = context.dctrl_parser
+    debputy_language_server.trust_language_ids = True
+    debputy_language_server.forced_locale = None
+
+    debputy_language_server.finish_startup_initialization()
+
+    return debputy_language_server
+
+
+def _fake_initialize(
+    debputy_ls: "DebputyLanguageServer",
+    doc_uri: str,
+    text: str,
+    *,
+    language_id: str = "",
+    version: int = 1,
+) -> None:
+    import lsprotocol.types as types
+
+    debputy_ls.lsp.lsp_initialize(
+        types.InitializeParams(
+            types.ClientCapabilities(
+                general=types.GeneralClientCapabilities(
+                    position_encodings=[types.PositionEncodingKind.Utf32],
+                )
+            )
+        )
+    )
+    debputy_ls.workspace.put_text_document(
+        types.TextDocumentItem(doc_uri, language_id, version, text)
+    )
+
+
 def ensure_lint_and_lsp_commands_are_loaded() -> None:
     # Loading the module does the heavy lifting
     # However, having this function means that we do not have an "unused" import that some tool
