@@ -12,7 +12,7 @@ import shutil
 import subprocess
 import tempfile
 import textwrap
-from contextlib import ExitStack
+from contextlib import ExitStack, suppress
 from tempfile import mkstemp
 from typing import (
     Iterable,
@@ -37,12 +37,8 @@ from typing import (
 import debian.deb822
 from debian.changelog import Changelog
 from debian.deb822 import Deb822
-
 from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
-from debputy.dh.debhelper_emulation import (
-    dhe_install_pkg_file_as_ctrl_file_if_present,
-)
 from debputy.elf_util import find_all_elf_files, ELF_MAGIC
 from debputy.exceptions import DebputyDpkgGensymbolsError, PureVirtualPathError
 from debputy.filesystem_scan import FSPath, FSControlRootDir, VirtualPathBase
@@ -51,6 +47,7 @@ from debputy.maintscript_snippet import (
     MaintscriptSnippetContainer,
     STD_CONTROL_SCRIPTS,
 )
+from debputy.packager_provided_files import PackagerProvidedFile
 from debputy.packages import BinaryPackage, SourcePackage
 from debputy.packaging.alternatives import process_alternatives
 from debputy.packaging.debconf_templates import process_debconf_templates
@@ -58,6 +55,7 @@ from debputy.packaging.makeshlibs import (
     compute_shlibs,
     ShlibsContent,
     generate_shlib_dirs,
+    resolve_reserved_provided_file,
 )
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.impl import ServiceRegistryImpl
@@ -72,7 +70,7 @@ from debputy.plugin.api.spec import (
     PackageProcessingContext,
     ServiceDefinition,
 )
-from debputy.plugin.debputy.binary_package_rules import ServiceRule
+from debputy.plugins.debputy.binary_package_rules import ServiceRule
 from debputy.util import (
     _error,
     ensure_dir,
@@ -141,24 +139,28 @@ def generate_md5sums_file(
 
 
 def install_or_generate_conffiles(
-    binary_package: BinaryPackage,
-    ctrl_root: str,
+    ctrl_root: Union[FSPath, FSControlRootDir],
     fs_root: VirtualPath,
-    debian_dir: VirtualPath,
+    reserved_packager_provided_files: Dict[str, List[PackagerProvidedFile]],
 ) -> None:
-    conffiles_dest = os.path.join(ctrl_root, "conffiles")
-    dhe_install_pkg_file_as_ctrl_file_if_present(
-        debian_dir,
-        binary_package,
+    provided_conffiles_file = resolve_reserved_provided_file(
         "conffiles",
-        ctrl_root,
-        0o0644,
+        reserved_packager_provided_files,
     )
+    if (
+        provided_conffiles_file
+        and provided_conffiles_file.is_file
+        and provided_conffiles_file.size > 0
+    ):
+        ctrl_root.insert_file_from_fs_path(
+            "conffiles",
+            provided_conffiles_file.fs_path,
+            mode=0o644,
+            reference_path=provided_conffiles_file,
+        )
     etc_dir = fs_root.lookup("etc")
     if etc_dir:
-        _add_conffiles(conffiles_dest, (p for p in etc_dir.all_paths() if p.is_file))
-    if os.path.isfile(conffiles_dest):
-        os.chmod(conffiles_dest, 0o0644)
+        _add_conffiles(ctrl_root, (p for p in etc_dir.all_paths() if p.is_file))
 
 
 PERL_DEP_PROGRAM = 1
@@ -1119,6 +1121,7 @@ def setup_control_files(
             fs_root,
             package_state.reserved_packager_provided_files,
             package_state.maintscript_snippets,
+            substvars,
         )
         process_debconf_templates(
             binary_package,
@@ -1245,10 +1248,9 @@ def setup_control_files(
 
     if allow_ctrl_file_management:
         install_or_generate_conffiles(
-            binary_package,
-            control_output_fs_path,
+            control_output_dir,
             fs_root,
-            manifest.debian_dir,
+            package_state.reserved_packager_provided_files,
         )
 
     _generate_control_files(
@@ -1288,16 +1290,19 @@ def _generate_snippet(
 
 
 def _add_conffiles(
-    conffiles_dest: str,
+    ctrl_root: VirtualPathBase,
     conffile_matches: Iterable[VirtualPath],
 ) -> None:
-    with open(conffiles_dest, "at") as fd:
-        for conffile_match in conffile_matches:
+    it = iter(conffile_matches)
+    first = next(it, None)
+    if first is None:
+        return
+    conffiles = itertools.chain([first], it)
+    with ctrl_root.open_child("conffiles", "at") as fd:
+        for conffile_match in conffiles:
             conffile = conffile_match.absolute
             assert conffile_match.is_file
             fd.write(f"{conffile}\n")
-    if os.stat(conffiles_dest).st_size == 0:
-        os.unlink(conffiles_dest)
 
 
 def _ensure_base_substvars_defined(substvars: FlushableSubstvars) -> None:
@@ -1580,15 +1585,27 @@ def dpkg_field_list_pkg_dep() -> Sequence[str]:
     return output.decode("utf-8").splitlines(keepends=False)
 
 
-def _handle_relationship_substvars(
+_SUBSTVARS_FIELDS_NOT_SUPPORTED_BY_DPKG = {
+    "Commands",
+}
+
+
+@functools.lru_cache()
+def all_auto_substvars() -> Sequence[str]:
+    result = [x for x in dpkg_field_list_pkg_dep()]
+    result.extend(_SUBSTVARS_FIELDS_NOT_SUPPORTED_BY_DPKG)
+    return tuple(result)
+
+
+def _handle_auto_substvars(
     source: SourcePackage,
     dctrl_file: BinaryPackage,
     substvars: FlushableSubstvars,
     has_dbgsym: bool,
 ) -> Optional[str]:
-    relationship_fields = dpkg_field_list_pkg_dep()
-    relationship_fields_lc = frozenset(x.lower() for x in relationship_fields)
-    substvar_fields = collections.defaultdict(list)
+    auto_substvars_fields = all_auto_substvars()
+    auto_substvars_fields_lc = {x.lower(): x for x in auto_substvars_fields}
+    substvar_fields = collections.defaultdict(set)
     needs_dbgsym_stanza = False
     for substvar_name, substvar in substvars.as_substvar.items():
         if ":" not in substvar_name:
@@ -1602,9 +1619,9 @@ def _handle_relationship_substvars(
             continue
         _, field = substvar_name.rsplit(":", 1)
         field_lc = field.lower()
-        if field_lc not in relationship_fields_lc:
+        if field_lc not in auto_substvars_fields_lc:
             continue
-        substvar_fields[field_lc].append("${" + substvar_name + "}")
+        substvar_fields[field_lc].add("${" + substvar_name + "}")
 
     if not has_dbgsym:
         needs_dbgsym_stanza = False
@@ -1614,7 +1631,7 @@ def _handle_relationship_substvars(
 
     replacement_stanza = debian.deb822.Deb822(dctrl_file.fields)
 
-    for field_name in relationship_fields:
+    for field_name in auto_substvars_fields:
         field_name_lc = field_name.lower()
         addendum = substvar_fields.get(field_name_lc)
         if addendum is None:
@@ -1629,6 +1646,20 @@ def _handle_relationship_substvars(
             existing_value = existing_value.rstrip().rstrip(",")
             final_value = f"{existing_value}, {substvars_part}"
         replacement_stanza[field_name] = final_value
+        canonical_field_name = auto_substvars_fields_lc.get(field_name_lc)
+        # If `dpkg` does not know the field, we need to inject `XB-` in front
+        # of it.
+        if (
+            canonical_field_name
+            and canonical_field_name in _SUBSTVARS_FIELDS_NOT_SUPPORTED_BY_DPKG
+        ):
+            replacement_stanza[f"XB-{canonical_field_name}"] = replacement_stanza[
+                field_name
+            ]
+            del replacement_stanza[field_name]
+
+    with suppress(KeyError):
+        replacement_stanza.order_last("Description")
 
     tmpdir = generated_content_dir(package=dctrl_file)
     with tempfile.NamedTemporaryFile(
@@ -1706,7 +1737,7 @@ def _generate_control_files(
         has_dbgsym = dbgsym_root_fs is not None and any(
             f for f in dbgsym_root_fs.all_paths() if f.is_file
         )
-        dctrl_file = _handle_relationship_substvars(
+        dctrl_file = _handle_auto_substvars(
             source_package,
             binary_package,
             substvars,
