@@ -19,6 +19,7 @@ from typing import (
     Tuple,
     Sequence,
     FrozenSet,
+    Callable,
 )
 
 from debian.debian_support import DpkgArchTable
@@ -34,7 +35,6 @@ from .architecture_support import DpkgArchitectureBuildProcessValuesTable
 from .builtin_manifest_rules import builtin_mode_normalization_rules
 from .exceptions import (
     DebputySubstitutionError,
-    DebputyRuntimeError,
     DebputyRuntimeErrorWithPreamble,
 )
 from .filesystem_scan import FSPath, FSRootDir, FSROOverlay, FSControlRootDir
@@ -75,8 +75,8 @@ from .plugin.api.spec import (
     INTEGRATION_MODE_DH_DEBPUTY_RRR,
     INTEGRATION_MODE_FULL,
 )
-from .plugin.debputy.binary_package_rules import ServiceRule
-from .plugin.debputy.to_be_api_types import BuildRule
+from debputy.plugins.debputy.binary_package_rules import ServiceRule
+from debputy.plugins.debputy.build_system_rules import BuildRule
 from .plugin.plugin_state import run_in_context_of_plugin
 from .substitution import Substitution
 from .transformation_rules import (
@@ -1133,6 +1133,25 @@ def _install_everything_from_source_dir_if_present(
         pass
 
 
+def _add_build_install_dirs_to_per_package_search_dirs(
+    build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
+    per_package_search_dirs: Dict[BinaryPackage, List[VirtualPath]],
+    as_path: Callable[[str], VirtualPath],
+) -> None:
+    seen_pp_search_dirs: Set[Tuple[BinaryPackage, str]] = set()
+    for dest_dir, for_packages in build_system_install_dirs:
+        dest_path = as_path(dest_dir)
+        for pkg in for_packages:
+            seen_key = (pkg, dest_dir)
+            if seen_key in seen_pp_search_dirs:
+                continue
+            seen_pp_search_dirs.add(seen_key)
+            if pkg not in per_package_search_dirs:
+                per_package_search_dirs[pkg] = [dest_path]
+            else:
+                per_package_search_dirs[pkg].append(dest_path)
+
+
 class HighLevelManifest:
     def __init__(
         self,
@@ -1238,6 +1257,7 @@ class HighLevelManifest:
     def perform_installations(
         self,
         integration_mode: DebputyIntegrationMode,
+        build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
         *,
         install_request_context: Optional[InstallSearchDirContext] = None,
     ) -> PackageDataTable:
@@ -1246,6 +1266,18 @@ class HighLevelManifest:
         enable_manifest_installation_feature = (
             integration_mode != INTEGRATION_MODE_DH_DEBPUTY_RRR
         )
+
+        if build_system_install_dirs:
+            if integration_mode != INTEGRATION_MODE_FULL:
+                raise ValueError(
+                    "The build_system_install_dirs parameter can only be used in full integration mode"
+                )
+            if install_request_context:
+                raise ValueError(
+                    "The build_system_install_dirs parameter cannot be used with install_request_context"
+                    " (not implemented)"
+                )
+
         if install_request_context is None:
 
             @functools.lru_cache(None)
@@ -1268,35 +1300,11 @@ class HighLevelManifest:
                 # Even dtmp_dir is omitted here (since it is not universally applicable).
                 # Note we still initialize dtmp_dir, since it affects a later guard.
                 default_search_dirs = []
-                seen_pp_search_dirs: Set[Tuple[BinaryPackage, str]] = set()
-                # TODO: We should **not** re-detect the default build system here. Instead,
-                #  we should have accumulated these per-package install dirs when we
-                #  invoked the build systems (notably, we are also not using the right
-                #  environment, which happens to work for now but that might break in
-                #  the future).
-                build_rules = self.build_rules
-                if build_rules is None:
-                    from .build_support.buildsystem_detection import (
-                        auto_detect_buildsystem,
-                    )
-
-                    bs = auto_detect_buildsystem(self)
-                    build_rules = [bs] if bs is not None else []
-
-                for build_rule in build_rules:
-                    dest_dir = build_rule.install_dest_dir()
-                    if dest_dir is None:
-                        continue
-                    dest_path = _as_path(dest_dir)
-                    for pkg in build_rule.for_packages:
-                        seen_key = (pkg, dest_dir)
-                        if seen_key in seen_pp_search_dirs:
-                            continue
-                        seen_pp_search_dirs.add(seen_key)
-                        if pkg not in per_package_search_dirs:
-                            per_package_search_dirs[pkg] = [dest_path]
-                        else:
-                            per_package_search_dirs[pkg].append(dest_path)
+                _add_build_install_dirs_to_per_package_search_dirs(
+                    build_system_install_dirs,
+                    per_package_search_dirs,
+                    _as_path,
+                )
 
                 # We can end here with per_package_search_dirs having no search dirs for any package
                 # (single binary, where everything is installed into d/<pkg> is the most common case).
