@@ -5,6 +5,7 @@ import datetime
 import os
 import time
 from collections import defaultdict, Counter
+from dataclasses import fields
 from enum import IntEnum
 from functools import lru_cache
 from typing import (
@@ -22,10 +23,13 @@ from typing import (
     Iterable,
     Awaitable,
     AsyncIterable,
+    Tuple,
+    TypeVar,
 )
 
+
 import debputy.l10n as l10n
-from debputy.commands.debputy_cmd.output import OutputStylingBase
+from debputy.commands.debputy_cmd.output import IOBasedOutputStyling
 from debputy.dh.dh_assistant import (
     extract_dh_addons_from_control,
     DhSequencerData,
@@ -35,6 +39,7 @@ from debputy.exceptions import PureVirtualPathError, DebputyRuntimeError
 from debputy.filesystem_scan import VirtualPathBase
 from debputy.integration_detection import determine_debputy_integration_mode
 from debputy.l10n import Translations
+from debputy.lsp.config.debputy_config import DebputyConfig
 from debputy.lsp.diagnostics import (
     LintSeverity,
     LINT_SEVERITY2LSP_SEVERITY,
@@ -47,6 +52,12 @@ from debputy.packages import SourcePackage, BinaryPackage
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.spec import DebputyIntegrationMode
 from debputy.util import _warn, T
+from debputy.lsp.vendoring._deb822_repro.locatable import (
+    Range as TERange,
+    Position as TEPosition,
+    Locatable,
+    START_POSITION,
+)
 
 if TYPE_CHECKING:
     import lsprotocol.types as types
@@ -55,14 +66,14 @@ if TYPE_CHECKING:
         MaintainerPreferenceTable,
         EffectiveFormattingPreference,
     )
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
-        Range as TERange,
-        Position as TEPosition,
-    )
+
 else:
     import debputy.lsprotocol.types as types
 
 
+L = TypeVar("L", bound=Locatable)
+
+
 AsyncLinterImpl = Callable[["LintState"], Awaitable[None]]
 FormatterImpl = Callable[["LintState"], Optional[Sequence[types.TextEdit]]]
 
@@ -102,6 +113,18 @@ def te_range_to_lsp(te_range: "TERange") -> types.Range:
     )
 
 
+def with_range_in_continuous_parts(
+    iterable: Iterable["L"],
+    *,
+    start_relative_to: "TEPosition" = START_POSITION,
+) -> Iterable[Tuple["TERange", "L"]]:
+    current_pos = start_relative_to
+    for part in iterable:
+        part_range = part.size().relative_to(current_pos)
+        yield part_range, part
+        current_pos = part_range.end_pos
+
+
 @lru_cache
 def _check_diagnostic_source(source: str) -> None:
     if source in DIAG_SOURCE_WITHOUT_SECTIONS:
@@ -154,6 +177,21 @@ class RelatedDiagnosticInformation:
         )
 
 
+@dataclasses.dataclass(frozen=True, slots=True)
+class WorkspaceTextEditSupport:
+    supports_document_changes: bool = False
+    supported_resource_operation_edit_kinds: Sequence[types.ResourceOperationKind] = (
+        dataclasses.field(default_factory=list)
+    )
+
+    @property
+    def supports_versioned_text_edits(self) -> bool:
+        return (
+            self.supports_document_changes
+            or self.supported_resource_operation_edit_kinds
+        )
+
+
 class LintState:
 
     @property
@@ -172,6 +210,10 @@ class LintState:
         """
         raise NotImplementedError
 
+    @property
+    def doc_version(self) -> Optional[int]:
+        raise NotImplementedError
+
     @property
     def source_root(self) -> Optional[VirtualPathBase]:
         """The path to the unpacked source root directory if available
@@ -271,15 +313,23 @@ class LintState:
         """
         raise NotImplementedError
 
+    @property
+    def workspace_text_edit_support(self) -> WorkspaceTextEditSupport:
+        raise NotImplementedError
+
+    @property
+    def debputy_config(self) -> DebputyConfig:
+        raise NotImplementedError
+
     def spellchecker(self) -> "Spellchecker":
         checker = default_spellchecker()
         ignored_words = set()
         source_package = self.source_package
         binary_packages = self.binary_packages
-        if source_package and source_package.fields.get("Source") is not None:
-            ignored_words.add(source_package.fields.get("Source"))
+        if source_package and (name := source_package.fields.get("Source")) is not None:
+            ignored_words.add(name)
         if binary_packages:
-            ignored_words.update(binary_packages)
+            ignored_words.update(binary_packages.keys())
         return checker.context_ignored_words(ignored_words)
 
     async def slow_iter(
@@ -407,6 +457,11 @@ class LintState:
         raise NotImplementedError
 
 
+CLI_WORKSPACE_TEXT_EDIT_SUPPORT = WorkspaceTextEditSupport(
+    supports_document_changes=True,
+)
+
+
 @dataclasses.dataclass(slots=True)
 class LintStateImpl(LintState):
     plugin_feature_set: PluginProvidedFeatureSet = dataclasses.field(repr=False)
@@ -416,6 +471,7 @@ class LintStateImpl(LintState):
     path: str
     content: str
     lines: List[str]
+    debputy_config: DebputyConfig
     source_package: Optional[SourcePackage] = None
     binary_packages: Optional[Mapping[str, BinaryPackage]] = None
     effective_preference: Optional["EffectiveFormattingPreference"] = None
@@ -430,6 +486,10 @@ class LintStateImpl(LintState):
         abs_path = os.path.join(os.path.curdir, path)
         return f"file://{abs_path}"
 
+    @property
+    def doc_version(self) -> Optional[int]:
+        return None
+
     @property
     def position_codec(self) -> "LintCapablePositionCodec":
         return LINTER_POSITION_CODEC
@@ -471,6 +531,10 @@ class LintStateImpl(LintState):
             self._dh_sequencer_cache = dh_sequencer_cache
         return dh_sequencer_cache
 
+    @property
+    def workspace_text_edit_support(self) -> WorkspaceTextEditSupport:
+        return CLI_WORKSPACE_TEXT_EDIT_SUPPORT
+
     async def gather_diagnostics(self) -> List[types.Diagnostic]:
         if self._diagnostics is not None:
             raise RuntimeError(
@@ -689,7 +753,7 @@ def debputy_severity(diagnostic: types.Diagnostic) -> LintSeverity:
 
 class TermLintReport(LintReport):
 
-    def __init__(self, fo: OutputStylingBase) -> None:
+    def __init__(self, fo: IOBasedOutputStyling) -> None:
         super().__init__()
         self.fo = fo
 
@@ -838,7 +902,7 @@ def _lines_to_print(range_: types.Range) -> int:
 
 
 def _highlight_range(
-    fo: OutputStylingBase,
+    fo: IOBasedOutputStyling,
     line: str,
     line_no: int,
     range_: types.Range,
