@@ -15,6 +15,7 @@ from typing import (
     Generic,
     Protocol,
     TYPE_CHECKING,
+    Literal,
 )
 
 from debputy.commands.debputy_cmd.context import CommandContext
@@ -45,6 +46,9 @@ SEMANTIC_TOKENS_LEGEND = types.SemanticTokensLegend(
         types.SemanticTokenTypes.Comment.value,
         types.SemanticTokenTypes.String.value,
         types.SemanticTokenTypes.Macro.value,
+        types.SemanticTokenTypes.Operator.value,
+        types.SemanticTokenTypes.TypeParameter.value,
+        types.SemanticTokenTypes.Variable.value,
     ],
     token_modifiers=[],
 )
@@ -64,7 +68,8 @@ DiagnosticHandler = Callable[
 @dataclasses.dataclass(slots=True)
 class LanguageDispatchTable(Generic[C]):
     language_id: str
-    filename_based_lookups: Dict[str, C] = dataclasses.field(default_factory=dict)
+    basename_based_lookups: Dict[str, C] = dataclasses.field(default_factory=dict)
+    path_name_based_lookups: Dict[str, C] = dataclasses.field(default_factory=dict)
     default_handler: Optional[C] = None
 
 
@@ -101,28 +106,37 @@ TEXT_DOC_INLAY_HANDLERS: HandlerDispatchTable[C] = HandlerDispatchTable()
 _ALIAS_OF = {}
 
 
+@dataclasses.dataclass(slots=True, frozen=True)
+class BasenameMatchingRule:
+    rule_type: Literal["basename", "extension"]
+    value: str
+
+
 @dataclasses.dataclass(slots=True, frozen=True)
 class SecondaryLanguage:
     language_id: str
-    filename_based_lookup: bool = False
+    secondary_lookup: Optional[Literal["path-name", "basename"]] = None
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class LanguageDispatchRule:
     primary_language_id: str
-    filenames: Sequence[str]
+    basename: Optional[str]
+    path_names: Sequence[str]
     secondary_language_ids: Sequence[SecondaryLanguage]
 
     @classmethod
     def new_rule(
         cls,
         primary_language_id: str,
-        filenames: Union[str, Sequence[str]],
+        basename: Optional[str],
+        path_names: Union[str, Sequence[str]],
         secondary_language_ids: Sequence[Union[SecondaryLanguage, str]] = (),
     ) -> Self:
         return LanguageDispatchRule(
             primary_language_id,
-            (filenames,) if isinstance(filenames, str) else tuple(filenames),
+            basename,
+            (path_names,) if isinstance(path_names, str) else tuple(path_names),
             tuple(
                 SecondaryLanguage(l) if isinstance(l, str) else l
                 for l in secondary_language_ids
@@ -243,24 +257,42 @@ def _register_handler(
         )
 
     primary_table.default_handler = handler
-    for filename in file_format.filenames:
-        filename_based_handler = filename_based_dispatch.filename_based_lookups.get(
+    for filename in file_format.path_names:
+        filename_based_handler = filename_based_dispatch.path_name_based_lookups.get(
             filename
         )
         if filename_based_handler is not None:
             raise AssertionError(f"There is already a handler for filename {filename}")
-        filename_based_dispatch.filename_based_lookups[filename] = handler
+        filename_based_dispatch.path_name_based_lookups[filename] = handler
 
     for secondary_language in file_format.secondary_language_ids:
         secondary_table = handler_dict[secondary_language.language_id]
-        if secondary_language.filename_based_lookup:
-            for filename in file_format.filenames:
-                secondary_handler = secondary_table.filename_based_lookups.get(filename)
+        if secondary_language.secondary_lookup == "path-name":
+            if not file_format.path_names:
+                raise AssertionError(
+                    f"secondary_lookup=path-name requires the language to have path-names. Please correct definition of {file_format.primary_language_id}"
+                )
+            for filename in file_format.path_names:
+                secondary_handler = secondary_table.path_name_based_lookups.get(
+                    filename
+                )
                 if secondary_handler is not None:
                     raise AssertionError(
                         f"There is already a handler for filename {filename} under language ID {secondary_language.language_id}"
                     )
-                secondary_table.filename_based_lookups[filename] = handler
+                secondary_table.path_name_based_lookups[filename] = handler
+        elif secondary_language.secondary_lookup == "basename":
+            basename = file_format.basename
+            if not basename:
+                raise AssertionError(
+                    f"secondary_lookup=basename requires the language to have a basename. Please correct definition of {file_format.primary_language_id}"
+                )
+            secondary_handler = secondary_table.basename_based_lookups.get(basename)
+            if secondary_handler is not None:
+                raise AssertionError(
+                    f"There is already a handler for basename {basename} under language ID {secondary_language.language_id}"
+                )
+            secondary_table.basename_based_lookups[basename] = handler
         elif secondary_table.default_handler is not None:
             raise AssertionError(
                 f"There is already a primary handler for language ID {secondary_language.language_id}"
@@ -270,11 +302,14 @@ def _register_handler(
 
 
 def ensure_lsp_features_are_loaded() -> None:
-    # FIXME: This import is needed to force loading of the LSP files. But it only works
-    #  for files with a linter (which currently happens to be all of them, but this is
-    #  a bit fragile).
+    # These imports are needed to force loading of the LSP files. The relevant registration
+    # happens as a side effect of the imports.
+    import debputy.lsp.languages as lsp_languages
     from debputy.linting.lint_impl import LINTER_FORMATS
 
+    # Ensure no static analysis tool is temped to optimize out the imports. We need them
+    # for the side effect.
+    assert lsp_languages
     assert LINTER_FORMATS
 
 
