@@ -1,5 +1,7 @@
 import dataclasses
 import os.path
+from importlib.resources.abc import Traversable
+from pathlib import Path
 from typing import (
     Optional,
     Callable,
@@ -68,7 +70,6 @@ from debputy.substitution import VariableContext
 from debputy.util import _normalize_path, package_cross_check_precheck
 
 if TYPE_CHECKING:
-    import lsprotocol.types as types
     from debputy.plugin.api.spec import (
         ServiceDetector,
         ServiceIntegrator,
@@ -79,11 +80,9 @@ if TYPE_CHECKING:
         PackageTransformationDefinition,
         BinaryPackageData,
     )
-    from debputy.plugin.debputy.to_be_api_types import (
+    from debputy.plugins.debputy.to_be_api_types import (
         BuildRuleParsedFormat,
     )
-else:
-    import debputy.lsprotocol.types as types
 
 
 TD = TypeVar("TD", bound="Union[DebputyParsedContent, List[DebputyParsedContent]]")
@@ -104,7 +103,13 @@ class DebputyPluginMetadata:
     plugin_loader: Optional[Callable[[], Callable[["DebputyPluginInitializer"], None]]]
     plugin_initializer: Optional[Callable[["DebputyPluginInitializer"], None]]
     plugin_path: str
+    plugin_doc_path_resolver: Callable[[], Optional[Union[Traversable, Path]]] = (
+        lambda: None
+    )
+    is_from_python_path: bool = False
     _is_initialized: bool = False
+    _is_doc_path_resolved: bool = False
+    _plugin_doc_path: Optional[Union[Traversable, Path]] = None
 
     @property
     def is_bundled(self) -> bool:
@@ -118,6 +123,13 @@ class DebputyPluginMetadata:
     def is_initialized(self) -> bool:
         return self._is_initialized
 
+    @property
+    def plugin_doc_path(self) -> Optional[Union[Traversable, Path]]:
+        if not self._is_doc_path_resolved:
+            self._plugin_doc_path = self.plugin_doc_path_resolver()
+            self._is_doc_path_resolved = True
+        return self._plugin_doc_path
+
     def initialize_plugin(self, api: "DebputyPluginInitializer") -> None:
         if self.is_initialized:
             raise RuntimeError("Cannot load plugins twice")
@@ -160,7 +172,9 @@ class PluginProvidedParser(Generic[PF, TP]):
         parser_context: "ParserContextData",
     ) -> TP:
         parsed_value = self.parser.parse_input(
-            value, attribute_path, parser_context=parser_context
+            value,
+            attribute_path,
+            parser_context=parser_context,
         )
         return self.handler(name, parsed_value, attribute_path, parser_context)
 
@@ -494,7 +508,8 @@ class DispatchingObjectParser(
         expected_debputy_integration_mode: Optional[
             Container[DebputyIntegrationMode]
         ] = None,
-        unknown_keys_diagnostic_severity: LintSeverity = "error",
+        unknown_keys_diagnostic_severity: Optional[LintSeverity] = "error",
+        allow_unknown_keys: bool = False,
     ) -> None:
         super().__init__(manifest_attribute_path_template)
         self._attribute_documentation: List[ParserAttributeDocumentation] = []
@@ -503,6 +518,7 @@ class DispatchingObjectParser(
         self._parser_documentation = parser_documentation
         self._expected_debputy_integration_mode = expected_debputy_integration_mode
         self._unknown_keys_diagnostic_severity = unknown_keys_diagnostic_severity
+        self._allow_unknown_keys = allow_unknown_keys
 
     @property
     def unknown_keys_diagnostic_severity(self) -> Optional[LintSeverity]:
@@ -601,7 +617,7 @@ class DispatchingObjectParser(
             )
         result = {}
         unknown_keys = orig_value.keys() - self._parsers.keys()
-        if unknown_keys:
+        if unknown_keys and not self._allow_unknown_keys:
             first_key = next(iter(unknown_keys))
             remaining_valid_attributes = self._parsers.keys() - orig_value.keys()
             if not remaining_valid_attributes:
@@ -1133,7 +1149,7 @@ def expand_known_packaging_config_features(
     return sorted(final_features)
 
 
-class InstallPatternDHCompatRule(DebputyParsedContent):
+class DHCompatibilityBasedRule(DebputyParsedContent):
     install_pattern: NotRequired[str]
     add_config_features: NotRequired[List[KnownPackagingConfigFeature]]
     starting_with_compat_level: NotRequired[int]
@@ -1150,7 +1166,7 @@ class KnownPackagingFileInfo(DebputyParsedContent):
     debhelper_commands: NotRequired[List[str]]
     config_features: NotRequired[List[KnownPackagingConfigFeature]]
     install_pattern: NotRequired[str]
-    dh_compat_rules: NotRequired[List[InstallPatternDHCompatRule]]
+    dh_compat_rules: NotRequired[List[DHCompatibilityBasedRule]]
     default_priority: NotRequired[int]
     post_formatting_rewrite: NotRequired[Literal["period-to-underscore"]]
     packageless_is_fallback_for_all_packages: NotRequired[bool]
